/**************************************************************************
 *   HEADER FILE : LOSDEFS.H - Contains definitions for the LOS program
 *
 *   AUTHOR      : Kevin Russo, SAIC (Original, 1)
 *
 *   Change     Date          Reason
 *   ------     ----          ------
 *   Original   01 Apr 93     n/a
 *   1          26 Jul 93     removed unused sections
 *
 **************************************************************************/

#ifndef LOS_DEFS_HEADER

#define LOS_DEFS_HEADER

/************************************************************************
 *                      Definitions
 ************************************************************************/

#define minof(A, B)  ((A) < (B) ? (A) : (B))
#define maxof(A, B)  ((A) > (B) ? (A) : (B))

#ifndef ACUITY_FACTOR
#define ACUITY_FACTOR     0.0002908333
#endif
#ifndef ARCTAN_DEGREES
#define ARCTAN_DEGREES    57.2957795
#endif
#ifndef BEYOND_ACUITY
#define BEYOND_ACUITY     5
#endif
#ifndef DEFATTR
#define DEFATTR           TX_BORDER|TX_NOCUT|TX_NOCOPY|TX_NOPASTE|TX_NOMENU|TX_ONEPAR
#endif
#ifndef DEF_HIDDEN
#define DEF_HIDDEN        4
#endif
#ifndef DEF_VISIBLE
#define DEF_VISIBLE       1
#endif
#ifndef DEG_PER_RAD
#define DEG_PER_RAD       57.2957795 
#endif
#ifndef EARTH_RADIUS
#define EARTH_RADIUS      6378137.0  
#endif
#ifndef ERRCOORD
#define ERRCOORD          -32767
#endif
#ifndef EXTENTS
#define EXTENTS           1.0
#endif
#ifndef F
#define F                 0
#endif
#ifndef FEET_PER_METER
#define FEET_PER_METER    3.2808
#endif
#ifndef GRID_VALUE
#define GRID_VALUE        0.99
#endif
#ifndef HEADER
#define HEADER            "Line of Sight Version 1.1 Product"
#endif
#ifndef HEADER_LEN
#define HEADER_LEN        33
#endif
#ifndef INIT_LOS_VAL
#define INIT_LOS_VAL     -99999L
#endif
#ifndef INIT_VERT_ACC
#define INIT_VERT_ACC     20L
#endif
#ifndef MAX_ANGLE
#define MAX_ANGLE         360.0
#endif
#ifndef MAX_RADIALS
#define MAX_RADIALS       2000
#endif
#ifndef MAX_ZOOM
#define MAX_ZOOM          3
#endif
#ifndef METERS_PER_DEGREE
#define METERS_PER_DEGREE 111096.0
#endif
#ifndef MIN_ANGLE
#define MIN_ANGLE         0.0
#endif
#ifndef MIN_SCREEN_HGT
#define MIN_SCREEN_HGT    480
#endif
#ifndef MIN_SCREEN_WID
#define MIN_SCREEN_WID    640
#endif
#ifndef MIN_ZOOM
#define MIN_ZOOM          1
#endif
#ifndef OBSERVER
#define OBSERVER          1
#endif
#ifndef ONE
#define ONE               1
#endif
#ifndef PI
#define PI                3.1415926535 
#endif
#ifndef PROB_HIDDEN
#define PROB_HIDDEN       3
#endif
#ifndef PROB_VISIBLE
#define PROB_VISIBLE      2
#endif
#ifndef T
#define T                 1
#endif
#ifndef TARGET
#define TARGET            2
#endif
#ifndef TAN_FACTOR
#define TAN_FACTOR        0.017453293
#endif
#ifndef UNDEFINED
#define UNDEFINED         -10
#endif
#ifndef UNKNOWN_DTED
#define UNKNOWN_DTED      -32767
#endif
#ifndef ZERO
#define ZERO              0
#endif

/************************************************************************
 *                      color definitions
 ************************************************************************/

#if XVT_OS == XVT_OS_DOS || XVT_OS == XVT_OS_WIN2 || XVT_OS == XVT_OS_WIN3
#define COLOR_DKGRN 0x0C004010L
#elif XVT_OS == XVT_OS_MAC
#ifdef  COLOR_LTGRAY
#undef  COLOR_LTGRAY
#define COLOR_LTGRAY 0x0CB0B0B0L
#endif
#define COLOR_DKGRN 0x0C005050L
#else
#define COLOR_DKGRN XVT_MAKE_COLOR (0x28, 0x86, 0x28)
#endif

#define COLOR_GRAY62 XVT_MAKE_COLOR (0xA0, 0xA0, 0xA0)

/************************************************************************
 *                      structures
 ************************************************************************/

typedef enum { AMSL, AGL } HGT_REL_TYPE;

typedef enum { DD_STYLE, DMS_STYLE } LOC_UNITS;

typedef enum { METERS, FEET } DIST_UNITS;

typedef enum { VISIBILITY, HIDDEN } PRIOR_MASK;

typedef enum { VIS, VIP } PROD_MASK;

typedef enum { C_RED, C_GREEN, C_BLUE, C_CYAN, C_MAGENTA,
               C_YELLOW, C_BLACK, C_WHITE } COL_TYPE;

typedef enum { S_CIRCLE, S_SQUARE } SHAPE_TYPE;

typedef struct {
   GFLOAT x;
   GFLOAT y;
   GFLOAT z;
} xyz_type;

typedef struct {
   GFLOAT x;
   GFLOAT y;
} coord_type;

typedef struct {
   coord_type loc;
   GFLOAT     height;
   GFLOAT     width;
} position_type;

typedef struct {
   HGT_REL_TYPE hgt_rel;
   LOC_UNITS    loc_units;
   DIST_UNITS   dist_units;
   BOOLEAN      acuity;
   SHAPE_TYPE   obs_shape;
   COL_TYPE     obs_col;
   SHAPE_TYPE   tgt_shape;
   COL_TYPE     tgt_col;
   COL_TYPE     bor_col;
} PREFS_TYPE;

typedef struct
{
   long red;
   long green;
   long blue;
} BININFO; 

typedef struct {
   GFLOAT       radial_spacing;      /* spaceing between radials          */
   GFLOAT       grid_size_m;         /* grid res  (meters)                */
   GFLOAT       grid_size_dd;        /* grid res (decimal degrees)        */
   GFLOAT       x_res_m;             /* grid lon res in meters            */
   GFLOAT       y_res_m;             /* grid lat res in meters            */
   GFLOAT       radial_step_size;    /* steps between radials             */
   GFLOAT       orig_radial_length;  /* original radial length            */
   GFLOAT       radial_length;       /* radial length                     */
   long         num_radials;         /* number of radials                 */
   long         nhts_in_radial;      /* number of radial heights          */
   long         max_nhts_in_radial;  /* maximum num of heights in radial  */
   long         radial_count;        /* radials counter                   */
} PARAM_CALC;

typedef struct {
   long  visible;                    /* total visible items               */
   long  prob_visible;               /* total prob visible items          */
   long  prob_hidden;                /* total prob hidden items           */
   long  hidden;                     /* total hidden items                */
   long  acuity;                     /* total outside acuity distance     */
   long  total;                      /* total items                       */
} PERCENT_TYPE;

typedef struct {
   GHANDLE  dist;                    /* distances handle                  */
   GHANDLE  hgts;                    /* heights handle                    */
   GHANDLE  xdim;                    /* elevation x-dimension handle      */
   GHANDLE  ydim;                    /* elevation y-dimension handle      */
   GHANDLE  elev;                    /* elevation matrix values handle    */
   GHANDLE  prfl;                    /* profile color handle              */
} HANDLES;

typedef struct {
   BOOLEAN vis_prior;                /* data win vis priority mask?       */
   BOOLEAN vis_prod;                 /* data win vis product mask?        */
} WIN_CTRL_STATUS;

typedef struct {
   GHANDLE shandle;                  /* handle to sine table              */
   GHANDLE chandle;                  /* handle to cosine table            */
} TRIG_TYPE;

typedef struct {
   position_type observer;           /* information about the observer    */
   position_type target;             /* information about the target      */
   PRIOR_MASK    prior_mask;         /* visibilty or hidden prior. mask   */
   PROD_MASK     prod_type;          /* VIS or VIP product type           */
   BOOLEAN       profile;            /* flag to check if profile prod.    */
   BOOLEAN       product;            /* flag to check if product exists   */
   GFLOAT        start;              /* starting azimuth angle in dec. deg*/
   GFLOAT        end;                /* ending azimuth angle in dec. deg. */
   GFLOAT        range;              /* range to target in meters         */
   long          vert_acc;           /* DTED vertical accuracy            */
} SETTINGS_TYPE;

typedef struct {
   BOOLEAN       boresight;          /* whether or not boresight is disp  */
   PNT           range;              /* basemap scroll bar range          */
   PNT           org;                /* basemap scroll bar slider position*/
   FILE_SPEC     dted_file;          /* file spec for basemap DTED file   */
   GFLOAT        dist;               /* distance to target in meters      */
   GFLOAT        prof_dist;          /* pofile distance to target in (m)  */
   coord_type    t;                  /* target latitude and longitude     */
   coord_type    o;                  /* observer latitude and longitude   */
   coord_type    cursor_pos;         /* position of the cursor within map */
   char          status[255];        /* basemap status message            */
   short         scale;              /* map scale (zoom factor ) - 1,2,3  */
} MAP_TYPE;

typedef struct {
   int size;                         /* system font height in pixels      */
   int leading;                      /* system font leader height         */
   int ascend;                       /* system font ascender height       */
   int descend;                      /* system font descender height      */
} FONT_SIZE_TYPE;

typedef struct {
   BOOLEAN  cancel_button_status;                 /* cancel button flag    */
   RCT      info_rect;                            /* info win in prof win  */
   long     lookupTable501[NUM_BASEMAP_CONTROLS]; /* basemap window id     */
   long     lookupTable502[NUM_DATA_CONTROLS];    /* data entry window id  */
   long     lookupTable503[NUM_PROFILE_CONTROLS]; /* profile window id     */
   long     lookupTable504[NUM_STATUS_CONTROLS];  /* status window id      */
} WIN_INFO;

#endif /* LOS_DEFS_HEADER */
