/*
 * Copyright 1994, 1995 Vectaport Inc., Cartoactive Systems, Cider Press
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <MapUnidraw/mapclasses.h>
#include <MapUnidraw/mapcomps.h>
#include <MapUnidraw/mapeditor.h>
#include <MapUnidraw/mapproj.h>
#include <MapUnidraw/mapstates.h>
#include <Unidraw/unidraw.h>
#include <IV-look/kit.h>
#include <InterViews/cursor.h>
#include <InterViews/session.h>
#include <InterViews/window.h>
#include <stdio.h>

struct pnode {
  char* name;
  char* params[5];
} plist[] = {
  "Miller Projection","proj=mill","ellps=WGS84","lon_0=0W","lat_0=0N","no_defs",
  "Mercator Projection","proj=merc","ellps=WGS84","lon_0=0W","no_defs","",
  "Robinson Projection","proj=robin","ellps=WGS84","lon_0=0W","no_defs","",
  "Polyconic Projection","proj=poly","ellps=WGS84","lon_0=0W","no_defs","",
  "Bonne Projection","proj=bonne","ellps=WGS84","lon_0=0W","lat_1=40N","no_defs",
#if !defined(sun)
  "Werner Projection","proj=bonne","ellps=WGS84","lon_0=0W","lat_1=90N","no_defs",
#endif
  "August Epicycloidal Projection","proj=august","ellps=WGS84","lon_0=0W","no_defs","",
  "Fahey Projection","proj=fahey","ellps=WGS84","lon_0=0W","no_defs","",
  "Foucaut Projection","proj=fouc","ellps=WGS84","lon_0=0W","no_defs","",
  "Foucaut Sinusoidal Projection","proj=fouc_s","ellps=WGS84","lon_0=0W","no_defs","",
  "Ginsburg VIII Projection","proj=gins8","ellps=WGS84","lon_0=0W","no_defs","",
  "Kavraisky V Projection","proj=kav5","ellps=WGS84","lon_0=0W","no_defs","",
  "Kavraisky VII Projection","proj=kav7","ellps=WGS84","lon_0=0W","no_defs","",
  "Larrivee Projection","proj=larr","ellps=WGS84","lon_0=0W","no_defs","",
  "Laskowski Projection","proj=lask","ellps=WGS84","lon_0=0W","no_defs","",
  "Nell Projection","proj=nell","ellps=WGS84","lon_0=0W","no_defs","",
  "Nell-Hammer Projection","proj=nell_h","ellps=WGS84","lon_0=0W","no_defs","",
  "Sinusoidal Projection","proj=sinu","ellps=WGS84","lon_0=0W","no_defs","",
  "Polyconic (American) Projection","proj=poly","ellps=WGS84","lon_0=0W","no_defs","",
  "Rectangular Polyconic Projection","proj=rpoly","ellps=WGS84","lon_0=0W","no_defs","",
  "Hammer Projection","proj=hammer","ellps=WGS84","lon_0=0W","no_defs","",
  "Wagner I Projection","proj=wag1","ellps=WGS84","lon_0=0W","no_defs","",
  "Wagner II Projection","proj=wag2","ellps=WGS84","lon_0=0W","no_defs","",
  "Wagner III Projection","proj=wag3","ellps=WGS84","lon_0=0W","no_defs","",
  "Wagner IV Projection","proj=wag4","ellps=WGS84","lon_0=0W","no_defs","",
  "Wagner V Projection","proj=wag5","ellps=WGS84","lon_0=0W","no_defs","",
  "Wagner VI Projection","proj=wag6","ellps=WGS84","lon_0=0W","no_defs","",
  "Wagner VII Projection","proj=wag7","ellps=WGS84","lon_0=0W","no_defs","",
  "Aitoff Projection","proj=aitoff","ellps=WGS84","lon_0=0W","no_defs","",
  "Winkel Tripel Projection","proj=wintri","ellps=WGS84","lon_0=0W","no_defs","",
  "Gall (Stereographic) Projection","proj=gall","ellps=WGS84","lon_0=0W","lat_0=0N","no_defs",
  "Equidistant Cylindrical Projection","proj=eqc","ellps=WGS84","lon_0=0W","lat_ts=45","no_defs",
  "Plate-Caree Projection","proj=eqc","ellps=WGS84","lon_0=0W","lat_ts=0","no_defs",
  "Mollweide Projection","proj=moll","ellps=WGS84","lon_0=0W","no_defs","",
  "Eckert I Projection","proj=eck1","ellps=WGS84","lon_0=0W","no_defs","",
  "Eckert II Projection","proj=eck2","ellps=WGS84","lon_0=0W","no_defs","",
  "Eckert III Projection","proj=eck3","ellps=WGS84","lon_0=0W","no_defs","",
  "Eckert IV Projection","proj=eck4","ellps=WGS84","lon_0=0W","no_defs","",
  "Eckert V Projection","proj=eck5","ellps=WGS84","lon_0=0W","no_defs","",
  "Eckert VI Projection","proj=eck6","ellps=WGS84","lon_0=0W","no_defs","",
  "Goode Homolosine Projection","proj=goode","ellps=WGS84","lon_0=0W","no_defs","",
  "Hatano Asymmetrical Equal-Area Projection","proj=hatano","ellps=WGS84","lon_0=0W","no_defs","",
  "Loximuthal Projection","proj=loxim","ellps=WGS84","lon_0=0W","lat_1=40n","no_defs",
  "McBryde-Thomas Flat-Polar Parabolic Projection","proj=mbtfpp","ellps=WGS84","lon_0=0W","no_defs","",
  "McBryde-Thomas Flat-Polar Quartic Projection","proj=mbtfpq","ellps=WGS84","lon_0=0W","no_defs","",
  "McBryde-Thomas Flat-Polar Sinusoidal Projection","proj=mbtfps","ellps=WGS84","lon_0=0W","no_defs","",
  "McBryde-Thomas Flat-Polar Sine (#1) Projection","proj=mbt_s","ellps=WGS84","lon_0=0W","no_defs","",
  "McBryde-Thomas Flat-Pole Sine (#2) Projection","proj=mbt_fps","ellps=WGS84","lon_0=0W","no_defs","",
  "Putnins P1 Projection","proj=putp1","ellps=WGS84","lon_0=0W","no_defs","",
  "Putnins P2 Projection","proj=putp2","ellps=WGS84","lon_0=0W","no_defs","",
  "Putnins P3 Projection","proj=putp3","ellps=WGS84","lon_0=0W","no_defs","",
  "Putnins P3' Projection","proj=putp3p","ellps=WGS84","lon_0=0W","no_defs","",
  "Putnins P4' Projection","proj=putp4p","ellps=WGS84","lon_0=0W","no_defs","",
  "Putnins P5 Projection","proj=putp5","ellps=WGS84","lon_0=0W","no_defs","",
  "Putnins P5' Projection","proj=putp5p","ellps=WGS84","lon_0=0W","no_defs","",
  "Putnins P6 Projection","proj=putp6","ellps=WGS84","lon_0=0W","no_defs","",
  "Putnins P6' Projection","proj=putp6p","ellps=WGS84","lon_0=0W","no_defs","",
  "Quartic Authalic Projection","proj=qua_aut","ellps=WGS84","lon_0=0W","no_defs","",
  "Winkel I Projection","proj=wink1","ellps=WGS84","lon_0=0W","no_defs","",
  "Werenskiold I Projection","proj=weren","ellps=WGS84","lon_0=0W","no_defs","",
  "Boggs Eumorphic Projection","proj=boggs","ellps=WGS84","lon_0=0W","no_defs","",
  "Collignon Projection","proj=collg","ellps=WGS84","lon_0=0W","no_defs","",
  "Denoyer Semi-Elliptical Projection","proj=denoy","ellps=WGS84","lon_0=0W","no_defs","",
  "Craster Parabolic Projection","proj=crast","ellps=WGS84","lon_0=0W","no_defs","",
  "Ortelius Oval Projection","proj=ortel","ellps=WGS84","lon_0=0W","no_defs","",
  "Null Projection", "", "", "", "", "",
#if 0
  "0W Lambert Azimuthal Equal Area Projection","proj=laea","ellps=WGS84","lon_0=0W","no_defs","",
  "45W Lambert Azimuthal Equal Area Projection","proj=laea","ellps=WGS84","lon_0=45W","no_defs","",
  "90W Lambert Azimuthal Equal Area Projection","proj=laea","ellps=WGS84","lon_0=90W","no_defs","",
  "135W Lambert Azimuthal Equal Area Projection","proj=laea","ellps=WGS84","lon_0=135W","no_defs","",
  "180W Lambert Azimuthal Equal Area Projection","proj=laea","ellps=WGS84","lon_0=180W","no_defs","",
  "225W Lambert Azimuthal Equal Area Projection","proj=laea","ellps=WGS84","lon_0=225W","no_defs","",
  "270W Lambert Azimuthal Equal Area Projection","proj=laea","ellps=WGS84","lon_0=270W","no_defs","",
  "315W Lambert Azimuthal Equal Area Projection","proj=laea","ellps=WGS84","lon_0=315W","no_defs","",
#endif
  nil
};

ClassId MapProjectionCmd::GetClassId () { return MAPPROJECTION_CMD; }

boolean MapProjectionCmd::IsA (ClassId id) {
    return MAPPROJECTION_CMD == id || Command::IsA(id);
}

MapProjectionCmd::MapProjectionCmd (ControlInfo* c) : Command(c) { Init(); }
MapProjectionCmd::MapProjectionCmd (Editor* ed) : Command(ed) { Init(); }

void MapProjectionCmd::Init() {
  _list = new StringList;
  String* str_i;
  for (int i=0; plist[i].name!=nil; i++) {
    str_i = new String(plist[i].name);
    _list->append(*str_i);
  }
  _chooser = new StrChooser(_list, new String("Select Projection:"),
			  WidgetKit::instance(),
			  Session::instance()->style());
  Resource::ref(_chooser);
}

Command* MapProjectionCmd::Copy () {
    Command* copy = new MapProjectionCmd(CopyControlInfo());
    InitCopy(copy);
    return copy;
}

void MapProjectionCmd::ProjectFClassComps(OverlayComp* comp) {
  if (comp->IsA(MAPFCLASS_COMP)) {
    comp->Interpret(this);
  }
  else if (comp->IsA(MAPFEATURE_COMP)) {

  }
  else if (comp->IsA(OVERLAYS_COMP)) {
    Iterator i;
    for (comp->First(i); !comp->Done(i); comp->Next(i)) {
      OverlayComp* c = (OverlayComp*)comp->GetComp(i);
      ProjectFClassComps(c);
    }
  }
}

void MapProjectionCmd::Execute () {
    MapEditor* editor = (MapEditor*)GetEditor();
    while (_chooser->post_for(editor->GetWindow())) {
        String str;
        if (_chooser->selected() > -1)
            str = _list->item_ref(_chooser->selected());
	else
	    continue;
        const char* name = str.string();
	int i;
	for (i=0;plist[i].name!=nil;i++) {
	    if (strcmp(name,plist[i].name)==0)
	        break;
	}
	PJ* p;
	if (strcmp(name, "Null Projection")==0)
	    p = nil;
	else
	if (!(p = pj_init(sizeof(plist[i].params)/sizeof(char*), plist[i].params))) {
	    fprintf(stderr, "Projection initialization failed\n");
	    exit(1);
	}
	editor->SetProjection(p);
	editor->projstate()->projection(plist[i].name,(const char**)plist[i].params);
        ((MapIdrawComp*)editor->GetComponent())->SetProjectionName(plist[i].name);
	editor->GetWindow()->cursor(hourglass);
	ProjectFClassComps((OverlayComp*)editor->GetComponent());
	unidraw->Update();
	break;
    }
}

void MapProjectionCmd::Default() {
  MapEditor* editor = (MapEditor*)GetEditor();
  PJ* p = pj_init(sizeof(plist[0].params)/sizeof(char*), plist[0].params);
  if (p == nil) {
    fprintf(stderr, "Projection initialization failed\n");
    exit(1);
  }
  editor->SetProjection(p);
  if (editor->projstate())
  editor->projstate()->projection(plist[0].name,(const char**)plist[0].params);
  ((MapIdrawComp*)editor->GetComponent())->SetProjectionName(plist[0].name);
}

void MapProjectionCmd::SetProjection(const char* name) {
  MapEditor* editor = (MapEditor*)GetEditor();
  int i;
  for (i=0;plist[i].name!=nil;i++) {
      if (strcmp(name,plist[i].name)==0)
	  break;
  }
  PJ* p;
  if (!(p = pj_init(sizeof(plist[i].params)/sizeof(char*), plist[i].params))) {
      fprintf(stderr, "Projection initialization failed\n");
      exit(1);
  }
  editor->SetProjection(p);
  editor->projstate()->projection(plist[i].name,(const char**)plist[i].params);
  ((MapIdrawComp*)editor->GetComponent())->SetProjectionName(plist[i].name);
}

boolean MapProjectionCmd::Reversible () { return false; }
