/*
 * Copyright 1994, 1995 Vectaport Inc., Cider Press
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#ifndef vpfcomps_h
#define vpfcomps_h

#include <MapUnidraw/mapcomps.h>
#include <MapUnidraw/mapviews.h>

#include <IV-2_6/_enter.h>

class VpfCoverage;
class VpfDatabase;
class VpfFClassScript;
class VpfFeatureClass;
class VpfLibrary;

class VpfFClassComp : public MapFClassComp {
public:
    VpfFClassComp(VpfFeatureClass*);
    VpfFClassComp(istream&);
    virtual ~VpfFClassComp();

    virtual void Interpret(Command*);
    virtual void Uninterpret(Command*);

    virtual Component* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);

    VpfFeatureClass* GetFeatureClass() { return (VpfFeatureClass*)_mapfc; }
    OverlaysComp** TileComps() { return _tcomps; }

protected:
    int _ntiles;
    OverlaysComp** _tcomps;
    char* _tmpname;
friend VpfFClassScript;
protected:
    ParamList* GetParamList();
    void GrowParamList(ParamList*);
    static ParamList* _vpf_fcl_params;
};

class VpfFClassView : public MapFClassView {
public:
    VpfFClassView(VpfFClassComp* = nil);
    virtual ~VpfFClassView();

    virtual void Interpret(Command*);
    virtual void Uninterpret(Command*);

    virtual Selection* SelectAll();
    virtual Selection* ViewContaining(Coord, Coord);
    virtual Selection* ViewsContaining(Coord, Coord);
    virtual Selection* ViewIntersecting(Coord, Coord, Coord, Coord);
    virtual Selection* ViewsIntersecting(Coord, Coord, Coord, Coord);
    virtual Selection* ViewsWithin(Coord, Coord, Coord, Coord);

  virtual void Update();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
    void IgnoreNone(boolean);

    void MaxMag(float);
    float MaxMag();
    virtual void AdjustForZoom(float factor, Coord cx, Coord cy);
    virtual void Hide();
    virtual void Show();
protected:
    boolean _ignore_none;
    float _maxmag;
    boolean _hidden;
};

class VpfCovComp : public MapCovComp {
public:
    VpfCovComp(VpfCoverage*);
    VpfCovComp(istream&);
    virtual ~VpfCovComp();

    virtual void Interpret(Command*);

    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);

    VpfCoverage* GetCoverage() { return (VpfCoverage*)_mapcov; }

protected:
    virtual MapFClassComp* CreateFClassComp(MapFeatureClass*);

protected:
    ParamList* GetParamList();
    void GrowParamList(ParamList*);
    static ParamList* _vpf_cov_params;
};

class VpfCovView : public MapCovView {
public:
    VpfCovView(VpfCovComp* = nil);

    virtual void Interpret(Command*);

    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
protected:
};

class VpfLibComp : public MapLibComp {
public:
    VpfLibComp(VpfLibrary*);
    VpfLibComp(istream&);
    virtual ~VpfLibComp();

    VpfLibrary* GetLibrary() { return (VpfLibrary*)_maplib; }

protected:
    virtual MapCovComp* CreateCovComp(MapCoverage*);

protected:
    ParamList* GetParamList();
    void GrowParamList(ParamList*);
    static ParamList* _vpf_lib_params;
};

class VpfDbComp : public MapDbComp {
public:
    VpfDbComp(VpfDatabase*);
    VpfDbComp(istream&);
    virtual ~VpfDbComp();

    void init();

    VpfDatabase* GetDatabase() { return (VpfDatabase*)_mapdb; }

  virtual void Configure(Editor*);
protected:
    virtual MapLibComp* CreateLibComp(MapLibrary*);

protected:
    ParamList* GetParamList();
    void GrowParamList(ParamList*);
    static ParamList* _vpf_db_params;
};

class VpfIdrawComp : public MapIdrawComp {
public:
    VpfIdrawComp();
    VpfIdrawComp(istream&);

    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);

    Component* Copy();

protected:
    ParamList* GetParamList();
    void GrowParamList(ParamList*);
    static ParamList* _vpf_idraw_params;
};

#include <IV-2_6/_leave.h>

#endif
